@@ -32,6 +32,7 @@
 #include <gmodule.h>
 #include <gst/gstobject.h>
 #include <gst/gstmacros.h>
+#include <gst/gststructure.h>
 
 G_BEGIN_DECLS
 
@@ -70,10 +71,17 @@ typedef enum
   GST_PLUGIN_ERROR_NAME_MISMATCH
 } GstPluginError;
 
-
+/**
+ * GstPluginFlags:
+ * @GST_PLUGIN_FLAG_CACHED: Temporarily loaded plugins
+ * @GST_PLUGIN_FLAG_BLACKLISTED: The plugin won't be scanned (again)
+ *
+ * The plugin loading state
+ */
 typedef enum
 {
-  GST_PLUGIN_FLAG_CACHED = (1<<0)
+  GST_PLUGIN_FLAG_CACHED = (1<<0),
+  GST_PLUGIN_FLAG_BLACKLISTED = (1<<1)
 } GstPluginFlags;
 
 /**
@@ -99,16 +107,35 @@ typedef enum {
 
 /**
  * GstPluginInitFunc:
- * @plugin: The plugin object that can be used to register #GstPluginFeatures for this plugin.
+ * @plugin: The plugin object
  *
  * A plugin should provide a pointer to a function of this type in the
  * plugin_desc struct.
- * This function will be called by the loader at startup.
+ * This function will be called by the loader at startup. One would then
+ * register each #GstPluginFeature.
  *
  * Returns: %TRUE if plugin initialised successfully
  */
 typedef gboolean (*GstPluginInitFunc) (GstPlugin *plugin);
 
+/**
+ * GstPluginInitFullFunc:
+ * @plugin: The plugin object
+ * @user_data: extra data
+ *
+ * A plugin should provide a pointer to a function of either #GstPluginInitFunc
+ * or this type in the plugin_desc struct.
+ * The function will be called by the loader at startup. One would then
+ * register each #GstPluginFeature. This version allows
+ * user data to be passed to init function (useful for bindings).
+ *
+ * Returns: %TRUE if plugin initialised successfully
+ *
+ * Since: 0.10.24
+ *
+ */
+typedef gboolean (*GstPluginInitFullFunc) (GstPlugin *plugin, gpointer user_data);
+
 /**
  * GstPluginDesc:
  * @major_version: the major version number of core that plugin was compiled for
@@ -208,6 +235,7 @@ struct _GstPluginClass {
  * The macro uses a define named PACKAGE for the #GstPluginDesc,source field.
  */
 #define GST_PLUGIN_DEFINE(major,minor,name,description,init,version,license,package,origin)	\
+G_BEGIN_DECLS \
 GST_PLUGIN_EXPORT GstPluginDesc gst_plugin_desc = {	\
   major,						\
   minor,						\
@@ -220,7 +248,8 @@ GST_PLUGIN_EXPORT GstPluginDesc gst_plugin_desc = {	\
   package,						\
   origin,						\
   GST_PADDING_INIT				        \
-};
+}; \
+G_END_DECLS
 
 /**
  * GST_PLUGIN_DEFINE_STATIC:
@@ -308,6 +337,18 @@ gboolean		gst_plugin_register_static	(gint major_version,
                                                          const gchar *package,
                                                          const gchar *origin);
 
+gboolean		gst_plugin_register_static_full	(gint major_version,
+                                                         gint minor_version,
+                                                         const gchar *name,
+                                                         gchar *description,
+                                                         GstPluginInitFullFunc init_full_func,
+                                                         const gchar *version,
+                                                         const gchar *license,
+                                                         const gchar *source,
+                                                         const gchar *package,
+                                                         const gchar *origin,
+                                                         gpointer user_data);
+
 G_CONST_RETURN gchar*	gst_plugin_get_name		(GstPlugin *plugin);
 G_CONST_RETURN gchar*	gst_plugin_get_description	(GstPlugin *plugin);
 G_CONST_RETURN gchar*	gst_plugin_get_filename		(GstPlugin *plugin);
@@ -316,6 +357,9 @@ G_CONST_RETURN gchar*	gst_plugin_get_license		(GstPlugin *plugin);
 G_CONST_RETURN gchar*	gst_plugin_get_source		(GstPlugin *plugin);
 G_CONST_RETURN gchar*	gst_plugin_get_package		(GstPlugin *plugin);
 G_CONST_RETURN gchar*	gst_plugin_get_origin		(GstPlugin *plugin);
+G_CONST_RETURN GstStructure*	gst_plugin_get_cache_data	(GstPlugin * plugin);
+void		gst_plugin_set_cache_data	(GstPlugin * plugin, GstStructure *cache_data);
+
 GModule *		gst_plugin_get_module		(GstPlugin *plugin);
 gboolean		gst_plugin_is_loaded		(GstPlugin *plugin);
 
